;;; Copyright © 2016 Efraim Flashner <efraim@flashner.co.il>
;;;
;;; This file is an addendum to GNU Guix.
;;;
;;; GNU Guix is free software; you can redistribute it and/or modify it
;;; under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 3 of the License, or (at
;;; your option) any later version.
;;;
;;; GNU Guix is distributed in the hope that it will be useful, but
;;; WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with GNU Guix.  If not, see <http://www.gnu.org/licenses/>.

(define-module (wip fdroid)
  #:use-module ((guix licenses) #:prefix license:)
  #:use-module (guix download)
  #:use-module (guix packages)
  #:use-module (guix build-system python)
  #:use-module (gnu packages check)
  #:use-module (gnu packages openstack)
  #:use-module (gnu packages ssh)
  #:use-module (gnu packages python)
  #:use-module (gnu packages python-crypto)
  #:use-module (gnu packages python-web)
  #:use-module (gnu packages version-control))

;; Broken tests...
(define-public python-gitdb2
  (package
    (name "python-gitdb2")
    (version "2.0.2")
    (source
      (origin
        (method url-fetch)
        (uri (pypi-uri "gitdb2" version))
        (sha256
         (base32
          "0w40ci6c5inmg5ciir563klg3n10q1npan28ljk307z9c5snvqzj"))))
    (build-system python-build-system)
    (arguments
     `(#:tests? #f))
    (propagated-inputs
     `(("python-smmap2" ,python-smmap2)))
    ;(native-inputs
    ; `(("python-nose" ,python-nose))) ; FIXME
    (home-page "https://github.com/gitpython-developers/gitdb")
    (synopsis "Git Object Database")
    (description "Git Object Database")
    (license license:bsd-3)))

(define-public python2-gitdb2
  (package-with-python2 python-gitdb2))

(define-public python-gitpython
  (package
    (name "python-gitpython")
    (version "2.1.5")
    (source
      (origin
        (method url-fetch)
        (uri (pypi-uri "GitPython" version))
        (sha256
         (base32
           "172xmf1bv4bizw0w246ajv0mrvmpziqpskqx70wx1cg2av9cn02w"))))
    (build-system python-build-system)
    (arguments
     `(#:tests? #f))
    (propagated-inputs
     `(("python-gitdb2" ,python-gitdb2)))
    (native-inputs
     `(("git" ,git)
       ("python-ddt" ,python-ddt)
       ("python-nose" ,python-nose)))
    (home-page "https://github.com/gitpython-developers/GitPython")
    (synopsis "Python Git Library")
    (description "This package provides a Python library for @code{git}.")
    (license license:bsd-3)))

(define-public python2-gitpython
  (package-with-python2 python-gitpython))

(define-public fdroidserver
  (package
    (name "fdroidserver")
    (version "0.7.0")
    (source
      (origin
        (method url-fetch)
        (uri (pypi-uri "fdroidserver" version))
        (sha256
         (base32
          "0vldch2yfdvgixd207z7zwwf4y65pjmcx2dhcpysaq3js416mrrx"))))
    (build-system python-build-system)
    (arguments
     `(#:phases
       (modify-phases %standard-phases
         (add-after 'unpack 'patch-data-prefix
           (lambda* (#:key outputs #:allow-other-keys)
             (substitute* "setup.py"
               (("data_prefix = sys\\.prefix")
                (string-append "data_prefix = '"
                               (assoc-ref outputs "out") "'")))
             #t)))))
    (inputs
     `(("python-apache-libcloud" ,python-apache-libcloud)
       ("python-clint" ,python-clint)
       ("python-gitpython" ,python-gitpython)
       ("python-mwclient" ,python-mwclient)
       ("python-paramiko" ,python-paramiko)
       ("python-pillow" ,python-pillow)
       ("python-pyasn1" ,python-pyasn1)
       ("python-pyasn1-modules" ,python-pyasn1-modules)
       ("python-pyyaml" ,python-pyyaml)
       ("python-requests" ,python-requests)))
    ;(propagated-inputs
    ; `(("python2-crypto" ,python-crypto))) ; not sure this is the right one
    (home-page "https://f-droid.org/")
    (synopsis "F-Droid Server Tools")
    (description "This provides F-Droid Server Tools.")
    (license license:agpl3+)))

;(define-public python-funcsigs
;  (package
;    (name "python-funcsigs")
;    (version "1.0.2")
;    (source
;      (origin
;        (method url-fetch)
;        (uri (pypi-uri "funcsigs" version))
;        (sha256
;         (base32
;          "0l4g5818ffyfmfs1a924811azhjj8ax9xd1cffr1mzd3ycn0zfx7"))))
;    (build-system python-build-system)
;    (arguments `(#:tests? #f)) ; tests fail to recongnize unittest2
;    (native-inputs
;     `(("python-setuptools" ,python-setuptools)))
;    (home-page "http://funcsigs.readthedocs.org")
;    (synopsis
;     "Python function signatures from PEP362 for Python 2.6, 2.7 and 3.2+")
;    (description
;     "Python function signatures from PEP362 for Python 2.6, 2.7 and 3.2+")
;    (license license:asl2.0)))
;
;(define-public python2-funcsigs
;  (package-with-python2 python-funcsigs))
;
;(define-public python-crypto
;  (package
;    (name "python-crypto")
;    (version "1.4.1")
;    (source
;      (origin
;        (method url-fetch)
;        (uri (pypi-uri "crypto" version))
;        (sha256
;         (base32
;          "1gd9cvw1j5j9nxhi7gla2pggnp678zj7l2dc8n4c2r82d9syjblg"))))
;    (build-system python-build-system)
;    (inputs
;     `(("python-naked" ,python-naked)
;       ("python-pyyaml" ,python-pyyaml)
;;       ("python-shellescape" ,python-shellescape)
;))
;    (home-page "https://github.com/chrissimpkins/crypto")
;    (synopsis "Simple symmetric GPG file encryption and decryption")
;    (description
;     "Simple symmetric GPG file encryption and decryption")
;    (license license:expat)))

;(define-public python2-crypto
;  (let ((base (package-with-python2 python-crypto)))
;    (package (inherit base)
;      (native-inputs
;       `(("python2-setuptools" ,python2-setuptools)
;         ,@(package-native-inputs base))))))

;(define-public python-shellescape
;  (package
;    (name "python-shellescape")
;    (version "3.4.1")
;    (source
;      (origin
;        (method url-fetch)
;        (uri (pypi-uri "shellescape" version))
;        (sha256
;         (base32
;          "0n5ky1b2vw2y0d4xl3qybyp2rk0gq5frjs8nr8ak6mgj2fyb4676"))))
;    (build-system python-build-system)
;    (home-page
;     "https://github.com/chrissimpkins/shellescape")
;    (synopsis
;     "Shell escape a string to safely use it as a token in a shell command (backport of Python shlex.quote for Python versions 2.x & < 3.3)")
;    (description
;     "Shell escape a string to safely use it as a token in a shell command (backport of Python shlex.quote for Python versions 2.x & < 3.3)")
;    (license license:expat)
;    (properties `((python2-variant . ,(delay python2-shellescape))))))

;(define-public python2-shellescape
;  (let ((base (package-with-python2
;                (strip-python2-variant python-shellescape))))
;    (package (inherit base)
;      (native-inputs
;       `(("python2-setuptools" ,python2-setuptools)
;         ,@(package-native-inputs base))))))
;
;(define-public python-naked
;  (package
;    (name "python-naked")
;    (version "0.1.31")
;    (source
;      (origin
;        (method url-fetch)
;        (uri (pypi-uri "Naked" version))
;        (sha256
;         (base32
;          "0zk793hh6z4lirfxz73z4f5yzyx8khhx5w92jh1hfpar2j56pdqj"))))
;    (build-system python-build-system)
;    (arguments
;     `(#:phases
;       (modify-phases %standard-phases
;         (add-after 'unpack 'fix-reference
;           ;; installing is much easier when you don't depend on yourself
;           (lambda _
;             (substitute* "setup.py" (("\\['Naked', ") "[")))))))
;    (inputs
;     `(("python-pyyaml" ,python-pyyaml)
;       ("python-requests" ,python-requests)))
;    (home-page "http://naked-py.com")
;    (synopsis "A command line application framework")
;    (description
;     "A command line application framework")
;    (license license:expat)
;    (properties `((python2-variant . ,(delay python2-naked))))))
;
;(define-public python2-naked
;  (let ((base (package-with-python2 (strip-python2-variant python-naked))))
;    (package (inherit base)
;      (native-inputs
;       `(("python2-setuptools" ,python2-setuptools)
;         ,@(package-native-inputs base))))))
