;; texlive-latex-base, texlive-latex-xcolor,  texlive-texmf-minimal, texlive-minimal
(define-module (wip lyx)
  #:use-module ((guix licenses) #:select (bsd-3 gpl2 gpl3 lgpl2.1 lgpl2.1+ lgpl3 x11-style gpl2+))
  #:use-module (guix packages)
  #:use-module (guix download)
  #:use-module (guix build utils)
  #:use-module (guix build-system cmake)
  #:use-module (guix build-system gnu)
  #:use-module (guix packages)
  #:use-module (guix utils)
  #:use-module (gnu packages)
  #:use-module (gnu packages algebra) ; for bc
  #:use-module (gnu packages boost)
  #:use-module (gnu packages compression)
  #:use-module (gnu packages libreoffice)  ; for hunspell
  #:use-module (gnu packages pkg-config)
  #:use-module (gnu packages python)
  #:use-module (gnu packages qt)
  #:use-module (gnu packages tex)
  #:use-module (gnu packages xorg))

; Directory /tmp/guix-build-lyx-2.2.3.drv-0/lyx-2.2.2/lib does not contain chkconfig.ltx.

;; on quitting: lassert.cpp (51): ASSERTION !formats.empty() VIOLATED IN graphics/GraphicsCacheItem.cpp:320

;; FIXME Just use cmake ??? option         If you use Qt 5, use -DLYX_USE_QT=QT5, otherwise Qt 4 will be searched., also LYX_EXTERNAL_BOOST
;; CMake options: LYX_ASPELL, LYX_ENCHANT, LYX_HUNSPELL, LYX_PACKAGE_SUFFIX, LYX_QUIET???, LYX_INSTALL_PREFIX???, LYX_EXTERNAL_BOOST, LYX_PROGRAM_SUFFIX???, just 1 for YES.

; configure:
;   --enable-monolithic-build[=LIST]
;                          Use monolithic build for modules in LIST (default:
;                          boost,client,insets,mathed,core,tex2lyx,frontend_qt)

;; FIXME create /home/dannym/.guix-profile/share/lyx/lyxmodules.lst
;; FIXME use multithreaded boost
;; FIXME add babel
;; FIXME unbundle lyx-2.2.2/3rdparty/: boost/  hunspell/  libiconv/ zlib/
;; FIXME check what mythes is and unbundle it.
;; FIXME Add debug output.
;; FIXME --with-extra-lib, --with-extra-inc, --with-extra-prefix.
;; FIXME Fix undefined reference to lyx::frontend::IconButton::staticMetaObject
;; FIXME checking whether STL is libstdc++ using the C++11 ABI... no
;; FIXME use enchant
(define-public lyxx
  (package
    (name "lyxx")
    (version "2.2.2")
    (source (origin
             (method url-fetch)
             (uri (string-append "http://ftp.lyx.org/pub/lyx/stable/2.2.x/" "lyx" "-" version ".tar.gz"))
             (sha256
              (base32
               "0i9sh29vjk3azh7ms7j9laxvwjif36vx353kw82i96vyxgp0lskz"))))
    (build-system gnu-build-system)
    (arguments
     `(#:configure-flags '("--without-included-boost"
                           "--enable-rpath" ; FIXME check whether it's actually correct.
                           "--enable-qt5"
                           "--enable-debug")
       #:phases
       (modify-phases %standard-phases
         (add-after 'configure 'patch-python
           (lambda* (#:key inputs #:allow-other-keys)
             (substitute* '("lib/configure.py"
                            "src/support/os.cpp"
                            "src/support/Systemcall.cpp"
                            "src/support/filetools.cpp"
                            "src/support/ForkedCalls.cpp"
                            "lib/scripts/include_bib.py")
               (("python -tt") "python3 -tt"))
             ;(setenv "QTDIR" (assoc-ref inputs "qt")) ; FIXME remove
             #t)))))
    (inputs
     `(("boost" ,boost)
       ("hunspell" ,hunspell) ; Note: could also use aspell instead.
       ("libx11" ,libx11)
       ("qt" ,qt)
       ("zlib" ,zlib)))
    (native-inputs
     `(("python" ,python)
       ("pkg-config" ,pkg-config)
       ("bc" ,bc)))
    (home-page "http://www.lyx.org/")
    (synopsis "Document preparation system with GUI")
    (description "LyX is a document preparation system.  It excels at letting you create complex technical and scientific articles with mathematics, cross-references, bibliographies, indexes, etc.  It is very good for working with documents of any length in which the usual processing abilities are required: automatic sectioning and pagination, spell checking and so forth. ")
    (license gpl2+)))

;; FIXME  DIFFSEARCHPATH=/etc/texmf pacdiff && fmtutil-sys --all
(define-public lyx
  (package
    (name "lyx")
    (version "2.2.3")
    (source (origin
             (method url-fetch)
             (uri (string-append "http://ftp.lyx.org/pub/lyx/stable/2.2.x/"
                                 name "-" version ".tar.gz"))
             (sha256
              (base32
               "0xvaz0i371nn2ndinc0d3ywj76ivb62649a4sdgwbivisiahd2fj"))))
    (build-system cmake-build-system)
    (arguments
     `(#:configure-flags '("-DLYX_USE_QT=QT5"
                           "-DLYX_EXTERNAL_BOOST=1"
                           ; -DLYX_ASPELL
                           ; -DLYX_ENCHANT
                           ; not found: "-DLYX_HUNSPELL=1"
                           ; -DLYX_QUIET???
                           ; -DLYX_INSTALL_PREFIX???
                           ; -DLYX_PACKAGE_SUFFIX=0
                           ; -DLYX_ASAN=1
                           ; -DLYX_PROGRAM_SUFFIX=0
)
       ;#:tests? #f ; or setenv LYX_DIR_22x=$(pwd) /lib
       #:phases
       (modify-phases %standard-phases
         (add-after 'unpack 'patch-python
           (lambda* (#:key inputs #:allow-other-keys)
             ;(substitute* '("lib/configure.py"
             ;               "src/support/os.cpp"
             ;               "src/support/Systemcall.cpp"
             ;               "src/support/filetools.cpp"
             ;               "src/support/ForkedCalls.cpp"
             ;               "lib/scripts/include_bib.py")
             ;  (("python -tt") "python3 -tt"))
             ;(substitute* "lib/configure.py"
             ;  (("print '''") "_ = '''") ; not that good.
             ;  (("print \"Unknown option\", op")
             ;   "print(\"Unknown option\", op)")
             ;  (("raise FileError,") "raise FileError(") ; )
             ;  (("\"cannot copy tree '%s': not a directory\" % src")
             ;   "\"cannot copy tree '%s': not a directory\" % src)") ;(
             ;  (("\"error listing files in '%s': %s\" % .src, errstr.")
             ;   "\"error listing files in '%s': %s\" % (src, errstr))"))
             ;(substitute* "src/tex2lyx/test/runtests.py"
             ;  (("print(.*)." _ args) ; last dot is the newline
             ;   (string-append "print(" args ")
;")))
;             (substitute* '("lib/scripts/lyxpreview_tools.py"
;                            "lib/configure.py")
;               (("[.]error,") ".error as ")  ; fix "except ..."
;               (("`sys.exc_type`") "sys.exc_type") ; WTF
;               (("`sys.exc_value`") "sys.exc_value") ; WTF
;               (("as  [(]errno, errstr[)]:") "as errstr:")) ; exception
; /tmp/guix-build-lyx-2.2.2.drv-0/build/Testing/.lyx/lyxrc.defaults missing!
;  File "/tmp/guix-build-lyx-2.2.2.drv-0/lyx-2.2.2/lib/scripts/../configure.py", line 120
;    raise FileError, \
;  File "/tmp/guix-build-lyx-2.2.2.drv-0/lyx-2.2.2/src/tex2lyx/test/runtests.py", line 100
;    print 'Executing: ' + cmd + "\n"

             ;(setenv "QTDIR" (assoc-ref inputs "qt")) ; FIXME remove
             #t))
         (add-before 'check 'setenv-check
           (lambda _
             (system* "pwd")
             (setenv "LYX_DIR_22x" (string-append (getcwd) "/../lyx-2.2.3/lib")) ; FIXME dynamic version
             #t)))))
    (inputs
     `(("boost" ,boost)
       ("hunspell" ,hunspell) ; Note: could also use aspell instead.
       ("libx11" ,libx11)
       ("qtbase" ,qtbase)
       ("qtsvg" ,qtsvg)
       ("zlib" ,zlib)
       ("texlive-tiny", texlive-tiny))) ; article.cls is in texmf-dist.
    (native-inputs
     `(("python-2" ,python-2)
       ("pkg-config" ,pkg-config)
       ("bc" ,bc)))
    (home-page "http://www.lyx.org/")
    (synopsis "Document preparation system with GUI")
    (description "LyX is a document preparation system.  It excels at letting you create complex technical and scientific articles with mathematics, cross-references, bibliographies, indexes, etc.  It is very good for working with documents of any length in which the usual processing abilities are required: automatic sectioning and pagination, spell checking and so forth. ")
    (license gpl2+)))
