;;; This file is an addendum to GNU Guix.
;;;
;;; GNU Guix is free software; you can redistribute it and/or modify it
;;; under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 3 of the License, or (at
;;; your option) any later version.
;;;
;;; GNU Guix is distributed in the hope that it will be useful, but
;;; WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with GNU Guix.  If not, see <http://www.gnu.org/licenses/>.

(define-module (wip openwrt)
  #:use-module ((guix licenses) #:prefix license:)
  #:use-module (guix git-download)
  #:use-module (guix packages)
  #:use-module (gnu packages cross-base)
  #:use-module (guix build-system gnu))

(define openwrt
  (package
    (name "openwrt")
    (version "0.6.0")
    (source
      (origin
        (method git-fetch)
        (file-name (string-append name "-" version "-checkout"))
        (uri (git-reference
              (url "git://github.com/openwrt/openwrt.git")
              (commit "c15bd8d6ee8bac11558c6d82bc8260ee8f20568d"))) ; branch chaos_calmer
        (sha256
         (base32
          "0bhasgriql3qz7ic3pkjg0jq6ynknj21jbdziw505jkyic1hjbqb"))))
    (build-system gnu-build-system)
    (home-page "https://openwrt.org/")
    (synopsis "OpenWRT")
    (description "OpenWRT.")
    (license license:agpl3+)))

;; TODO  run make menuconfig and set "Target System", "Subtarget", "Target Profile";
(define (make-openwrt-package board triplet)
  "Returns a openwrt package for BOARD cross-compiled for TRIPLET."
  (package
    (inherit openwrt)
    (name (string-append "openwrt-" (string-downcase board)))
    (native-inputs
     `(("cross-gcc" ,(cross-gcc triplet))
       ("cross-binutils" ,(cross-binutils triplet))
       ,@(package-native-inputs openwrt)))
    (arguments
     `(#:modules ((ice-9 ftw) (guix build utils) (guix build gnu-build-system))
       #:test-target "test"
       #:make-flags
       (list "HOSTCC=gcc" (string-append "CROSS_COMPILE=" ,triplet "-"))
       #:phases
       (modify-phases %standard-phases
         (replace 'configure
           (lambda* (#:key outputs make-flags #:allow-other-keys)
             (let ((config-name (string-append ,board "_defconfig"))) ; FIXME generate config file; then make defconfig
               (if (file-exists? (string-append "configs/" config-name))
                   (zero? (apply system* "make" `(,@make-flags ,config-name)))
                   (begin
                     (display "Invalid board name.")
                     (newline)
                     #f)))))
         (replace 'install
           (lambda* (#:key outputs make-flags #:allow-other-keys)
             (let* ((out (assoc-ref outputs "out"))
                    (libexec (string-append out "/libexec"))
                    (openwrt-files (find-files "." ".*\\.(bin|efi|spl)$")))
               (mkdir-p libexec)
               (for-each
                (lambda (file)
                  (let ((target-file (string-append libexec "/" file)))
                    (mkdir-p (dirname target-file))
                    (copy-file file target-file)))
                openwrt-files)))))))))

(define-public openwrt-vexpress
  (make-openwrt-package "vexpress_ca9x4" "arm-linux-gnueabihf"))

; openwrt-cups
; make package/cups/compile V=s

; TODO u-boot
