(define-module (wip pascal)
  #:use-module ((guix licenses) #:prefix license:)
  #:use-module (guix packages)
  #:use-module (guix download)
  #:use-module (guix git-download)
  #:use-module (guix hg-download)
  #:use-module (guix utils)
  #:use-module (guix build-system python)
  #:use-module (guix build-system gnu)
  #:use-module (guix gexp)
  #:use-module (gnu packages)
  #:use-module (gnu packages base)
  #:use-module (gnu packages gcc)
  #:use-module (gnu packages llvm)
  #:use-module (gnu packages multiprecision)
  #:use-module (gnu packages pkg-config))

(define-public gpc
  (let ((base (custom-gccx gcc-4.7 "gpc" '("pascal"))))
    (package
      (inherit base)
      (name "gpc")
      (native-inputs
       `(("tar" ,tar)
; REQUIRED: bash, bzip2, GNU sed, GNU awk, GNU m4, bison (at least version 2.0), flex (version 2.5.27), autoconf (version 2.12), texinfo (at least version 4.2), help2man.
; ENV before configure:
;          FLEX=/path/to/flex
;          LEX=/path/to/flex
;          BISON=/path/to/bison
;          YACC=/path/to/bison
;          INTLBISON=/path/to/bison
         ("gpc"
          ,(origin
             (method url-fetch)
             (uri "http://www.gnu-pascal.de/alpha/gpc-20060325.tar.bz2")
             (sha256
              (base32
               "1vadkpwm0087knb2lx48fk7mkv646ivwm84iy42qjw8z7svdrz8q"))))
          ,@(package-native-inputs base)))
      (arguments
       (substitute-keyword-arguments (package-arguments base)
        ((#:phases phases)
         `(modify-phases ,phases
           (add-after 'unpack 'unpack-submodule-sources
            (lambda* (#:key inputs #:allow-other-keys)
             (let ((unpack (lambda (source target)
                             (with-directory-excursion target
                               (zero? (system* "tar" "xvf"
                                               (assoc-ref inputs source)
                                               "--strip-components=1"))))))
              (unpack "gpc" "gcc")))))))))))

(define-public pure
  (package
    (name "pure")
    (version "0.66")
    (source
      (origin
        (method url-fetch)
        (uri (string-append "https://bitbucket.org/purelang/pure-lang/downloads/" name "-" version ".tar.gz"))
               (sha256
                (base32
                 "0xp6gq8x4bz61px6445phaydgvf1hh96fyd1ghxfx2vf8wr6ips2"))))
    (build-system gnu-build-system)
    (inputs
     `(("gmp" ,gmp)
       ("mpfr" ,mpfr)
       ("llvm" ,llvm-3.6)))
    (license license:lgpl3+)
    (home-page "https://purelang.bitbucket.io/")
    (synopsis "Pure programming language")
    (description "A programming language based on term rewriting.")))
