
(define-module (wip python)
  #:use-module ((guix licenses) #:prefix license:)
  #:use-module (guix packages)
  #:use-module (guix download)
  #:use-module (guix git-download)
  #:use-module (guix hg-download)
  #:use-module (guix utils)
  #:use-module (guix build-system python)
  #:use-module (guix build-system gnu)
  #:use-module (guix gexp)
  #:use-module (gnu packages)
  #:use-module (gnu packages base)
  #:use-module (gnu packages check)
  #:use-module (gnu packages compression)
  #:use-module (gnu packages databases)
  #:use-module (gnu packages libffi)
  #:use-module (gnu packages libusb)
  #:use-module (gnu packages linux)
  #:use-module (gnu packages ncurses)
  #:use-module (gnu packages libftdi)
  #:use-module (gnu packages tcl)
  #:use-module (gnu packages tls)
  #:use-module (gnu packages pkg-config)
  #:use-module (gnu packages python)
  #:use-module (gnu packages python-crypto)
  #:use-module (gnu packages python-web)
  #:use-module (gnu packages swig)
  #:use-module (gnu packages xml))

(define-public python-lazyasd
 (package
  (name "python-lazyasd")
  (version "0.1.3")
  (source
    (origin
      (method url-fetch)
;     BROKEN (uri (pypi-uri "lazyasd" version))
      (uri (string-append
             "https://pypi.python.org/packages/a7/c2/e32ac63f76031bccfbae08e2f20a1c7ad5f04431392ad1f53c43922c421f/lazyasd-"
             version
             ".tar.gz"))

      (sha256
        (base32
          "1znywqay5a42dh4nxcg1vb6mymgcfs352nzayiprmhvlzlk8xj77"))))
  (build-system python-build-system)
  (home-page "https://github.com/xonsh/lazyasd")
  (synopsis
    "Lazy & self-destructive tools for speeding up Python module imports")
  (description
    "This packages contains tools for speeding up Python module imports.")
  (license license:bsd-3)))

(define-public exofrills
  (package
   (name "exofrills")
   (version "0.1.13")
   (source
     (origin
       (method url-fetch)
       (uri (pypi-uri name version))
       (sha256
         (base32
           "15994d2f27mq4kfc1h8gvzy9xcyhn6n0syj0d2cjikkr4kgqa3i6"))))
   (build-system python-build-system)
   (inputs
     `(("python-setuptools" ,python-setuptools)
       ("python-lazyasd" ,python-lazyasd)
       ("python-urwid" ,python-urwid)
       ("python-pygments" ,python-pygments)))
   (home-page "http://exofrills.org/")
   (synopsis "exofrills text editor")
   (description "exofrills minimalist text editor")
   (license (license:non-copyleft "http://www.wtfpl.net/txt/copying/"))))

(define-public xonshxx
  (package
    (name "xonshxx")
    (version "0.4.6")
    (source
      (origin
        (method url-fetch)
        (uri (string-append "https://github.com/xonsh/xonsh/archive/"
                            version ".tar.gz"))
        (sha256
          (base32
            "0v0l3bpyxh2fyhybycjr22sh9v0ggswgmbm9gsyf7yvkrcr5fqzr"))
        (modules '((guix build utils)))
        (snippet
         `(begin
            ;; Delete bundled ply.
            (delete-file-recursively "xonsh/ply")
            (substitute* '("setup.py")
              (("'xonsh\\.ply',") ""))
            #t))))
    (build-system python-build-system)
    (arguments
     `(#:phases
       (modify-phases %standard-phases
         (add-after 'build 'build-docs
           (lambda* (#:key outputs #:allow-other-keys)
             (let ((out (assoc-ref outputs "out")))
               ;; Make it find "xonsh" module
               (setenv "PYTHONPATH" (string-append (getcwd) ":" (getenv "PYTHONPATH")))
               (zero? (system* "make" "-C" "docs" "html")))))
         (add-after 'install 'install-docs
           (lambda* (#:key outputs #:allow-other-keys)
             (let* ((out (assoc-ref outputs "out"))
                    (docout (string-append out "/doc"))
                    (htmlhelpout (string-append out "/doc/html")))
               (mkdir-p htmlhelpout)
               (copy-recursively "docs/_build/html" htmlhelpout)))))))
    (native-inputs
     `(("python-numpydoc" ,python-numpydoc)
       ("python-sphinx" ,python-sphinx)
       ("python-sphinx-cloud-sptheme"
        ,python-sphinx-cloud-sptheme)))
    (inputs
      ;; TODO jupyter distro
     `(("python-ply" ,python-ply)
       ("python-prompt-toolkit" ,python-prompt-toolkit)))
    (home-page "http://xon.sh/")
    (synopsis "Python-ish shell")
    (description
     "Xonsh is a Python-ish, BASHwards-looking shell language and command
prompt. The language is a superset of Python 3.4+ with additional shell
primitives that you are used to from Bash and IPython. It works on all major
systems including Linux, Mac OSX, and Windows. Xonsh is meant for the daily
use of experts and novices alike.")
    (license license:bsd-2)))

;; FIXME bpython [OK] for python-flask-script

(define-public python-platinum
  (let ((commit "28f5dcc994baba5f5925e3d7cc4b67997473e1a9"))
    (package
      (name "python-platinum")
      (version (string-append "0.0.1-" (string-take commit 7)))
      (source ;(origin
              ;  (method git-fetch)
              ;  (uri (git-reference
              ;        (url "ssh://git@stash.gmi.oeaw.ac.at:7999/hpc/platinum.git")
              ;        (commit commit)))
              ;  (file-name (string-append name "-" version "-checkout"))
              ;  (sha256
              ;   (base32
              ;    "10djsq5vlnkilv436gnmh4irpk49v29pa69r6xiryg32xmvn909j")))
              (local-file "/home/dannym/src/platinum/platinum" #:recursive? #t))
      (build-system python-build-system)
      (arguments
       `(#:phases
         (modify-phases %standard-phases
           (replace 'check
             (lambda _
               (chdir "webservice")
               (setenv "PLATINUM_DATABASE_URI" "sqlite://")
               (system* "touch" "./foo")
               (setenv "PLATINUM_CONFIG" (string-append (getcwd) "/foo"))
               ;; FIXME use a container with a running postgres server in it? FIXME re-enable
               (chdir "..")
               #t ; (zero? (system* "behave"))
)))))
      (native-inputs
        `(("postgresql" ,postgresql) ; for the tests
          ("python-behave-web-api" ,python-behave-web-api) ; for the tests
          ("python-alembic" ,python-alembic) ; only when migrating
          ("python-flask-script" ,python-flask-script)
          ("python-flask-migrate" ,python-flask-migrate)
          ("python-flask-principal" ,python-flask-principal)
          ;;("python-flask-restful-swagger" ,python-flask-restful-swagger)
          ("python-sadisplay" ,python-sadisplay) ; for docs
          ("python-swagger-spec-validator" ,python-swagger-spec-validator) ; for tests and docs
          ("behave" ,behave)))
      (propagated-inputs
        `(("python-docopt" ,python-docopt) ; for pcli
          ("python-simplejson" ,python-simplejson) ; for the hooks
          ("python-flask" ,python-flask)
          ;("python-xdg" ,python-xdg)
          ;; TODO use flask-restplus
          ;("python-flask-restplus" ,python-flask-restplus)
          ("python-flask-restful" ,python-flask-restful)
          ("python-flask-oidc" ,python-flask-oidc)
          ("python-flasgger" ,python-flasgger)
          ; old: ("python-flask-restful-swagger" ,python-flask-restful-swagger)
          ("python-flask-sqlalchemy" ,python-flask-sqlalchemy)
          ;("python-flask-basicauth" ,python-flask-basicauth)
          ;("python-flask-htpasswd" ,python-flask-htpasswd)
          ("python-sqlalchemy" ,python-sqlalchemy)
          ("python-sqlalchemy-utils" ,python-sqlalchemy-utils) ; For UUID at runtime; otherwise only when migrating
          ("python-psycopg2" ,python-psycopg2) ; used by sqlalchemy on our behalf
          ("python-pyopenssl" ,python-pyopenssl) ; used by werkzeug
          ("python-werkzeug" ,python-werkzeug)
          ("python-requests" ,python-requests) ; cli
          ("python-requests-oauthlib" ,python-requests-oauthlib))) ; cli
      (home-page "http://example.com/")
      (synopsis "Cluster accounting")
      (description "This is an accounting system for clusters. It provides a
REST API and requires a relational database (for example Postgres).")
      (license #f))))

; FIXME install flup FastCGI access.

(define-public pypy3
  (package
    (name "pypy3")
    (version "2.4.0")
    (source (origin
              (method url-fetch)
              (uri (string-append "https://bitbucket.org/pypy/pypy/downloads/"
                                  "pypy3-" version "-src.tar.bz2"))
              (sha256
               (base32
                "1wq4y3lmpiyapnkc8vncx11x3axi5plz9s8l8kfa1y7cdryj1fnr"))
              ;(patches (search-patches "pypy3.3-compiler-env.patch"))
))

    (build-system gnu-build-system)
    ;; TODO libgc for the untranslated tests
    (native-inputs
     ;; Note: python2-minimal doesn't work because ctypes.c_long is missing.
     `(("python-2" ,python-2)
       ("pkg-config" ,pkg-config)
       ("tar" ,tar)
       ("gzip" ,gzip)))
    (inputs
     `( ; for some reason this is not needed. ("bash" ,bash) ; actually /bin/sh
       ("libffi" ,libffi)
       ("zlib" ,zlib)
       ("ncurses" ,ncurses)
       ("openssl" ,openssl)
       ("expat" ,expat)
       ("bzip2" ,bzip2)
       ("sqlite" ,sqlite)
       ("gdbm" ,gdbm)
       ("tcl" ,tcl)
       ("tk" ,tk)
       ("xz" ,xz))) ; lzma
    (arguments
      `( ;#:modules ((srfi srfi-1)
          ;         (guix build utils))
        #:parallel-tests? #f ; FIXME is that necessary
        #:phases (modify-phases %standard-phases
         (delete 'configure)
         (replace 'build
            (lambda* (#:key inputs #:allow-other-keys)
              ;; distutils is used to verify extensions.
              ;; However, distutils build_ext uses a compiler that can only
              ;; be specified via CC environment variable.
              ;; Should have been used by customize_compiler() but wasn't.
              ;; Therefore we patch as well (in addition to setting it here).
              (setenv "CC" "gcc")
              ;; TODO set CPPFLAGS to something useful.
              ;; TODO set LDFLAGS rpath and other stuff
              (setenv "CCSHARED" "-fPIC")
               ;; TODO set LDFLAGS -L for tcl, tk, sqlite and rpath (mostly for the verification)
              (substitute* '("rpython/rtyper/lltypesystem/ll2ctypes.py")
                ;; Python ctypes.util.find_library documentation specifies
                ;; that it returns a file path, however all the actual
                ;; implementation cases return a SONAME.
                ;; Pypy expects it to return a file path.
                ;; Therefore, hotpatch ctypes.util to disable SONAME
                ;; determination.
                (("    import ctypes.util") "    import ctypes.util
    ctypes.util._get_soname = lambda x: x"))
              (substitute* '("lib-python/3/subprocess.py")
                (("\"/bin/sh\"") (string-append "\"" (which "sh") "\"")))
              (substitute* '("lib-python/3/distutils/log.py") ; FIXME remove
                (("__init__.self, threshold=WARN.") "__init__(self, threshold=INFO)")
                (("_global_log.threshold = level") "_x = level"))
              (substitute* '("ctypes_configure/cbuild.py" "lib_pypy/_pypy_testcapi.py")
                (("(compiler = new_compiler\\([^)]*\\))" all s1) (string-append s1 "; import distutils.sysconfig ; distutils.sysconfig.customize_compiler(compiler)
")))
              ;; FIXME just use main Makefile
              (chdir "pypy/goal") ; FIXME error checking
              (and
                (zero? (system* "python" "../../rpython/bin/rpython"
                                "-Ojit" "targetpypystandalone"))
                ;; Build out-of-line FFI modules (FIXME Fix failures in audioop etc)
                (zero? (system* "./pypy-c" "../../pypy/tool/build_cffi_imports.py")))))
         (replace 'check
            (lambda* (#:key inputs outputs #:allow-other-keys)
              (chdir "../..")
              (system* "pwd")
              (system* "ls") ; FIXME remove
              (display (getcwd))
              (setenv "LC_ALL" "en_US.utf8") ; make sure Python Unicode works
              (substitute* '("lib-python/3/test/test_shutil.py")
                ;; Make sure that the tests don't try to use uid / grp
                ;; (because we have no passwd entries in the container).
                (("UID_GID_SUPPORT = True") "UID_GID_SUPPORT = False")
                (("#!/bin/sh") (string-append "#!" (which "sh"))))
              (substitute* '("lib-python/3/test/test_posixpath.py")
                ;; Make sure that the tests don't try to use $HOME
                (("def test_expanduser") "
@unittest.skipIf(True, \"Disabled on GNU Guix\")
def test_expanduser"))
              (setenv "PYTHONPATH" (string-append (getcwd) ":"
                                                  ; very bad idea; would pick up the wrong grp module; (getcwd) "/lib_pypy:" ; for '_testcapi'
                                                  (getenv "PYTHONPATH")))
              ; FIXME maybe (add-installed-pythonpath inputs outputs) ; probably only useful AFTER installation.
              (and
                ;; TODO: " ./pypy/test_all.py"
                ;;       "lib-python/2.7.0/test/test_complex.py"
                ;;       "--pypy=foobar"
                (zero? (system* "mkdir" "/tmp/pytest"))
                ;; FIXME runs no tests:
                (zero? (system* "python" "-S", "testrunner/runner.py"
                                "--config=pypy/pytest-A.cfg" ; FIXME logfile is mandatory!
                                "--config=pypy/pytest-A.py"
                                "--logfile=/tmp/pytest/runner.log"
                                (string-append "--root=" (getcwd))
                                "--timeout=3600")) ; --config=~/machine-A_cfg.py
                ;; FIXME Fix test failures because it can't find _testcapi _audioop_cffi etc
                (zero? (system* "python" "-S", "pypy/test_all.py"
                                "--pypy=pypy/goal/pypy-c" "--timeout=3600"
                                "lib-python"))
                ;; All pass.
                (zero? (system* "python" "-S", "pypy/test_all.py"
                                "--pypy=pypy/goal/pypy-c" ; timeout doesn't work here
                                "pypy/module/pypyjit/test"))
                ;; FIXME Fix test failure 'No module named cStringIO'
                (zero? (system* "pypy/goal/pypy-c" "pypy/test_all.py"
                                "pypy/module/pypyjit/test_pypy_c")))))
         (replace 'install
            (lambda* (#:key outputs #:allow-other-keys)
              (chdir "../tool/release")
              ; TODO package.py --archive-name foobar
              (zero? (system* "./package.py pypy-VER-PLATFORM")))))))
    (home-page "http://www.pypy.org/")
    (synopsis "Python 3.x JIT")
    (description "This package provides a Python 3.x just-in-time compiler.")
    (license (list license:expat
                   license:psfl
                   license:asl2.0
                   license:gpl2+
                   license:bsd-2
                   license:bsd-3
                   (license:non-copyleft
                    "http://www.unicode.org/copyright.html")))))

(define-public pypy3.3
  (package (inherit pypy3)
    (name "pypy3.3")
    (version "5.5.0")
    (source (origin
              (method url-fetch)
              (uri (string-append "https://bitbucket.org/pypy/pypy/downloads/"
                                  "pypy3.3-v" version "-alpha-src.tar.bz2"))
              (sha256
               (base32
                "0pxbfmz5l3k3x3q23qf315y5zfc5jpsbd0niaznyjlvjsws1qnfm"))
              (patches (search-patches "pypy3.3-compiler-env.patch"))
              ;(method hg-fetch)
              ;(uri (hg-reference
              ;       (url "http://bitbucket.org/pypy/pypy")
              ;       ; FIXME branch py3.5; 89215:
              ;       (changeset "743a0fcaa4eb")))
              ;(sha256
              ; (base32
              ;  "0pxbfmz5l3k3x3q23qf315y5zfc5jpsbd0niaznyjlvjsws1qnfm"))
              (file-name (string-append "pypy3.5-v"
                                        version
                                        "-alpha1-src.tar.bz2"))))
    (arguments
      (substitute-keyword-arguments (package-arguments pypy3)
        ((#:phases phases)
        `(modify-phases ,phases
           (add-before 'build 'patch-3.3
             (lambda* (#:key inputs #:allow-other-keys)
              ;; FIXME Fix type mismatch 1.8.4 != 1.4.2
              (substitute* '("lib_pypy/_tkinter/tklib_build.py")
                (("/usr/include/tcl") (string-append (assoc-ref inputs "tcl") "/include"))
                (("'tcl' [+] _ver, 'tk' [+] _ver") "'tcl8.6', 'tk8.6'") ; FIXME dyna version
                (("libdirs = .*$") (string-append "libdirs = ['" (assoc-ref inputs "tcl") "/lib']
")))
              (substitute* '("lib_pypy/_sqlite3_build.py")
                (("libraries=.'sqlite3'..$") (string-append "libraries= ['sqlite3'], "
                                                           "include_dirs=['" (assoc-ref inputs "sqlite")  "/include'], "
                                                           "library_dirs=['" (assoc-ref inputs "sqlite") "/lib']"))
                (("if _has_load_extension..") "if True"))
            #t))))))))

(define-public python-watchdog
(package
  (name "python-watchdog")
  (version "0.8.3")
  (source
    (origin
      (method url-fetch)
      (uri (pypi-uri "watchdog" version))
      (sha256
        (base32
          "0qj1vqszxwfx6d1s66s96jmfmy2j94bywxiqdydh6ikpvcm8hrby"))))
  (build-system python-build-system)
  (propagated-inputs
    `(("python-setuptools" ,python-setuptools)))
  (home-page
    "http://github.com/gorakhargosh/watchdog")
  (synopsis "Filesystem events monitoring")
  (description "Filesystem events monitoring")
  (license #f)))

(define-public coconut
(package
  (name "coconut")
  (version "1.2.0")
  (source
    (origin
      (method url-fetch)
      (uri (pypi-uri "coconut" version))
      (sha256
        (base32
          "0c07v1f8ss4nz0c2r9id3n94y10gmvwxpmix35lm66pfa2hjqsyr"))))
  (build-system python-build-system)
  (propagated-inputs
    `(("python-ipython" ,python-ipython)
      ; FIXME ("python-jupyter" ,python-jupyter)
      ("python-jupyter-console"
       ,python-jupyter-console)
      ("python-psutil" ,python-psutil)
      ("python-setuptools" ,python-setuptools)
      ("python-watchdog" ,python-watchdog)))
  (home-page "http://coconut-lang.org")
  (synopsis
    "Simple, elegant, Pythonic functional programming.")
  (description
    "Simple, elegant, Pythonic functional programming.")
  (license #f)))

(define-public python-pyjwt ; use itsdangerous instead? Can't do RS256, though.
 (package
  (name "python-pyjwt")
  (version "1.5.0")
  (source
    (origin
      (method url-fetch)
      (uri (pypi-uri "PyJWT" version))
      (sha256
        (base32
          "0jy5sxaajm0l00c80xx6nk4mdlsnjb8269lvkll4rw0kimr2n67x"))))
  (build-system python-build-system)
  ;(propagated-inputs
  ; `(("python-cryptography" ,python-cryptography))) ; optional
  (native-inputs
   `(("python-pytest" ,python-pytest-3.0)
     ("python-pytest-cov" ,python-pytest-cov)
     ("python-pytest-runner" ,python-pytest-runner)))
  (home-page "http://github.com/jpadilla/pyjwt")
  (synopsis
    "JSON Web Token implementation in Python")
  (description
    "JSON Web Token implementation in Python")
  (license license:expat)))

(define-public python-flask-jwt
(package
  (name "python-flask-jwt")
  (version "0.3.2")
  (source
    (origin
      (method url-fetch)
      (uri (pypi-uri "Flask-JWT" version))
      (sha256
        (base32
          "1bfh7cr0sf65rn4h1q67472y6ml9s8c4k0xxfhrwvwg0plpngh29"))))
  (build-system python-build-system)
  (propagated-inputs
   `(("python-flask" ,python-flask)
     ("python-pyjwt" ,python-pyjwt)))
  (native-inputs
   `(("python-pytest" ,python-pytest-3.0)))
  (home-page "https://github.com/mattupstate/flask-jwt")
  (synopsis "JWT token authentication for Flask apps")
  (description "JWT token authentication for Flask apps")
  (license license:expat)))

; TODO galpy https://galpy.readthedocs.io/en/latest/reference/potentialisochrone.html

; xmlsec doesn't actually work at runtime! (load error)
(define-public python-xmlsec
(package
  (name "python-xmlsec")
  (version "1.0.7")
  (source
    (origin
      (method url-fetch)
      (uri (pypi-uri "xmlsec" version))
      (sha256
        (base32
          "12nghl8xs69fpjvivmxamkv21bmy0hsnrlz6am55f67a6z3v15q7"))))
  (build-system python-build-system)
  (inputs
   `(("xmlsec" ,xmlsec)
     ; FIXME missing ("libltdl" ,libltdl)
     ))
  (propagated-inputs
   `(("python-lxml" ,python-lxml)))
  (native-inputs
   `(("python-pkgconfig" ,python-pkgconfig) ; FIXME maybe move to propagated-inputs
     ("pkg-config" ,pkg-config))) ; FIXME remove and make it work.
  (home-page
    "https://github.com/mehcode/python-xmlsec")
  (synopsis
    "Python bindings for the XML Security Library")
  (description
    "Python bindings for the XML Security Library")
  (license license:expat)))

; Note: python2-saml is its own project <https://github.com/onelogin/python-saml/archive/v2.2.1.tar.gz>.
(define-public python-saml
(package
  (name "python-saml")
  (version "1.2.3")
  (source
    (origin
      (method url-fetch)
      (uri (string-append "https://github.com/onelogin/python3-saml/archive/v" version ".tar.gz"))
      (sha256
        (base32
          "0xac46qms1v1xwkfca0xv527chzn8bh42wlf07ycwjps1qqr1mmh"))))
  (propagated-inputs
   `( ;("python-dateutil" ,python-dateutil)
     ;("python-isodate" ,python-isodate)
     ;("python-xmlsec" ,python-xmlsec)
     ))
  (native-inputs
   `(("python-pkgconfig" ,python-pkgconfig))) ; FIXME move to python-xmlsec.
  (build-system python-build-system)
  (home-page
    "http://github.com/mehcode/python-saml")
  (synopsis
    "A python interface to produce and consume Security Assertion Markup Language (SAML) v2.0 messages.")
  (description
    "A python interface to produce and consume Security Assertion Markup Language (SAML) v2.0 messages.")
  (license license:expat))) ; FIXME which MIT

(define-public python-social-auth-core
 (package
  (name "python-social-auth-core")
  (version "1.2.0")
  (source
    (origin
      (method url-fetch)
      (uri (pypi-uri "social-auth-core" version))
      (sha256
        (base32
          "1p9n6azh33v099njv5plpj0a4ars499qd78mpnj0y1igii7287ki"))))
  (build-system python-build-system)
  (propagated-inputs
    `( ;("python-oauthlib" ,python-oauthlib)
      ("python-openid" ,python-openid)
      ("python-pyjwt" ,python-pyjwt)
      ("python-requests" ,python-requests)
      ("python-requests-oauthlib"
       ,python-requests-oauthlib)
      ("python-saml" ,python-saml) ; optional
      ("python-six" ,python-six)))
  (home-page
    "https://github.com/python-social-auth/social-core")
  (synopsis
    "Python social authentication")
  (description
    "Python social authentication made simple.")
  (license license:bsd-3)))

(define-public python-social-auth-app-flask
 (package
  (name "python-social-auth-app-flask")
  (version "1.0.0")
  (source
    (origin
      (method url-fetch)
      (uri (pypi-uri "social-auth-app-flask" version))
      (sha256
        (base32
          "0hphjnbibb02533bzyc3abwdrkk347pd63qzmwid1whvgw6dir6x"))))
  (build-system python-build-system)
  (propagated-inputs
    `( ;("python-flask-login" ,python-flask-login)
      ("python-six" ,python-six)
      ("python-social-auth-core"
       ,python-social-auth-core)))
  (home-page
    "https://github.com/python-social-auth/social-app-flask")
  (synopsis
    "Python Social Authentication, Flask integration.")
  (description
    "Python Social Authentication, Flask integration.")
  (license license:bsd-3)))

(define-public python-oidc-register
 (package
  (name "python-oidc-register")
  (version "0.0.1")
  (source
    (origin
      (method url-fetch)
      (uri (string-append
             "https://pypi.python.org/packages/ce/2d/f071411d17dff7808485ea277edc3fd2c50c7b04a1d8a77cd3c75d8fe39e/oidc-register-"
             version
             ".tar.gz"))
      (sha256
        (base32
          "1d19xahr2jszxqr1qg9gnkslxcpj9zj5dkvb6wqwqiywiz1f97nk"))))
  (build-system python-build-system)
  (arguments
   `(#:phases
     (modify-phases %standard-phases
       (add-after 'unpack 'remove-six
         (lambda _
           (substitute* "setup.py"
             (("'six',") ""))
         #t)))))
  ;(propagated-inputs
  ; `(("python-httplib2" ,python-httplib2))) ; vanished; FIXME
  (native-inputs
   `( ("python-nose" ,python-nose)
      ("python-mock" ,python-mock)
     ))
  (home-page
    "https://github.com/puiterwijk/oidc-register")
  (synopsis
    "OpenID Connect Dynamic Client Registration tool")
  (description
    "OpenID Connect Dynamic Client Registration tool")
  (license license:bsd-2)))

(define-public python-pyelftools
(package
  (name "python-pyelftools")
  (version "0.24")
  (source
    (origin
      (method url-fetch)
      (uri (pypi-uri "pyelftools" version))
      (sha256
        (base32
          "17259kf6hwwsmizr5myp9jv3k9g5i3dvmnl8m646pfd5hpb9gpg9"))))
  (build-system python-build-system)
  (arguments
   `(#:phases
     (modify-phases %standard-phases
       (add-before 'check 'set-pythonpath
         (lambda _
           ;; It doesn't find the "utils" module otherwise.
           (setenv "PYTHONPATH" (string-append (getcwd) "/test"))
           #t)))))
  (home-page
    "https://github.com/eliben/pyelftools")
  (synopsis
    "Library for analyzing ELF files and DWARF debugging information")
  (description
    "Library for analyzing ELF files and DWARF debugging information")
  (license license:public-domain)))

(define-public python-construct
(package
  (name "python-construct")
  (version "2.8.12")
  (source
    (origin
      (method url-fetch)
      (uri (pypi-uri "construct" version))
      (sha256
        (base32
          "1yhg4iwz8r0zmhi7wlrj15rlpxvfy47xxq670l3xrnhvl5ljrvk7"))))
  (build-system python-build-system)
  (native-inputs
   `(("python-pytest" ,python-pytest)
     ("python-pytest-cov" ,python-pytest-cov)
     ("python-numpy" ,python-numpy)))
  (arguments
   `(#:phases
     (modify-phases %standard-phases
       (replace 'check
         (lambda _
           (setenv "PYTHONPATH"
                   (string-append (getcwd) ":" (getenv "PYTHONPATH")))
           (zero? (system* "py.test" "--cov" "construct"
                                     "--cov-report" "html"
                                     "--cov-report" "term"
                                     "--verbose")))))))
  (home-page "http://construct.readthedocs.org/")
  (synopsis
    "A powerful declarative parser/builder for binary data")
  (description
    "A powerful declarative parser/builder for binary data")
  (license license:expat)))

(define-public python-pyrsp
  (package
  (name "python-pyrsp")
  (version "0.3.2")
  (source
    (origin
      (method url-fetch)
      (uri (pypi-uri "pyrsp" version))
      (sha256
        (base32
          "1p7g7bhri5gxiivrvim7c2lnhja2kkyx13spv9qq47y8h538j5vh"))))
  (build-system python-build-system)
  (propagated-inputs
   `(("python-pyelftools" ,python-pyelftools)
     ("python-pyserial" ,python-pyserial)
     ("python-construct" ,python-construct)))
  (home-page "https://github.com/stef/pyrsp")
  (synopsis
    "simple GDB remote serial protocol wrapper")
  (description
    "simple GDB remote serial protocol wrapper")
  (license license:gpl3+)))

(define-public python-usbtmc
  (package
    (name "python-usbtmc")
    (version "0.8")
    (source
     (origin
       (method url-fetch)
       (uri (string-append "https://github.com/alexforencich/python-usbtmc/archive/v" version ".tar.gz"))
       (sha256
        (base32
         "14f4j77ljr45crnjwlp1dqbxwa45s20y2fpq5rg59r60w15al4yw"))))
    (build-system python-build-system)
    (propagated-inputs
     `(("python-pyusb" ,python-pyusb)))
    (home-page "http://alexforencich.com/wiki/en/python-usbtmc/start")
    (synopsis "Oscilloscope library")
    (description "Osc.")
    (license license:expat)))

(define-public python2-xpybutil
  (let ((commit "c2d438d4ac246b60e25e1f04da2db0eab40acda5"))
    (package
      (name "python2-xpybutil")
      (version (string-append "0.0-1." (string-take commit 7)))
      (source
       (origin
        (method git-fetch)
        (uri (git-reference
              (url "https://github.com/BurntSushi/xpybutil.git")
              (commit commit)))
        (file-name (string-append name "-" version "-checkout"))
        (sha256
         (base32
          "0cq9y9wxyz3s4fjq6vgs27bpm5r2wg2s3nbsnjfp6fnfmp6q1gn1"))))
      (build-system python-build-system)
      (propagated-inputs
       `(("python2-xcffib" ,python2-xcffib)))
      (arguments
       `(#:python ,python-2))
      (home-page "https://github.com/BurntSushi/pytyle3")
      (synopsis "Program that tiles windows support lib FIXME")
      (description "@code{xpybutils} is a program that tiles windows XXX.")
      (license license:wtfpl2))))

(define-public python2-pytyle3
  (let ((commit "8a1ca4189659a4fd2250eb8c2166b630210e8eae"))
    (package
      (name "python2-pytyle3")
      (version (string-append "0.0-1." (string-take commit 7)))
      (source
       (origin
        (method git-fetch)
        (uri (git-reference
              (url "https://github.com/BurntSushi/pytyle3.git")
              (commit commit)))
        (file-name (string-append name "-" version "-checkout"))
        (sha256
         (base32
          "0vd1j82zw29vhn9366n8dgq1wv3jfxsr63lhkdj1w0x3hcmy3msq"))))
      (build-system python-build-system)
      (propagated-inputs
       `(("python2-xpybutil" ,python2-xpybutil)))
      (arguments
       `(#:python ,python-2
         #:phases
         (modify-phases %standard-phases
           (add-after 'unpack 'patch-installation-dir
             (lambda _
               (substitute* "setup.py"
                (("'/etc/xdg") "'etc/xdg"))
               #t)))))
      (home-page "https://github.com/BurntSushi/pytyle3")
      (synopsis "Program that tiles windows")
      (description "@code{pytyle3} is a program that tiles windows.")
      (license license:wtfpl2))))

; TODO merge to master:
(define-public python-pytest-pylint
  (package
    (name "python-pytest-pylint")
    (version "0.7.1")
    (source
      (origin
        (method url-fetch)
        (uri (pypi-uri "pytest-pylint" version))
        (sha256
          (base32
            "03914yd6j9ghsyzl64a8c1nix9fsnjk568blfjr18p9my146kcib"))))
    (build-system python-build-system)
    (propagated-inputs
     `(("python-pylint" ,python-pylint)
       ("python-pytest" ,python-pytest)))
    (home-page "https://github.com/carsongee/pytest-pylint")
    (synopsis "pytest plugin to check source code with pylint")
    (description "@code{pytest-pylint} is a @code{pytest} plugin to check
source code with @code{pylint}.")
    (license license:expat)))

(define-public python2-pytest-pylint
  (package-with-python2 python-pytest-pylint))

(define-public python-pathtools
  (package
    (name "python-pathtools")
    (version "0.1.2")
    (source
      (origin
        (method url-fetch)
        (uri (pypi-uri "pathtools" version))
        (sha256
          (base32
            "1h7iam33vwxk8bvslfj4qlsdprdnwf8bvzhqh3jq5frr391cadbw"))))
    (build-system python-build-system)
    (native-inputs
     `(("python-sphinx" ,python-sphinx)))
    (arguments
     `(#:tests? #f)) ; Not included and no github release available
    (home-page "http://github.com/gorakhargosh/pathtools")
    (synopsis "File system general utilities")
    (description "File system general utilities")
    (license license:expat))) ; FIXME which MIT?

(define-public python2-pathtools
  (package-with-python2 python-pathtools))

(define-public python-iocapture
  (package
    (name "python-iocapture")
    (version "0.1.2")
    (source
      (origin
        (method url-fetch)
        (uri (pypi-uri "iocapture" version))
        (sha256
          (base32
            "1s3ywdr0l3kfrrqi079iv16g0rp75akkvx0j07vx9p5w10c0wrw6"))))
    (build-system python-build-system)
    (native-inputs
     `(("python-flexmock" ,python-flexmock)
       ("python-pytest" ,python-pytest)
       ("python-pytest-cov" ,python-pytest-cov)
       ("python-six" ,python-six)))
    (home-page "https://github.com/oinume/iocapture")
    (synopsis "Captures stdout, stderr")
    (description "@code{iocapture} captures stdout, stderr.")
    (license license:expat)))

(define-public python2-iocapture
  (package-with-python2 python-iocapture))

(define-public python-argh
  (package
    (name "python-argh")
    (version "0.26.2")
    (source
      (origin
        (method url-fetch)
        (uri (pypi-uri "argh" version))
        (sha256
          (base32
            "0rdv0n2aa181mkrybwvl3czkrrikgzd4y2cri6j735fwhj65nlz9"))))
    (build-system python-build-system)
    (native-inputs
     `(("python-pytest" ,python-pytest)
       ;("python-mock" ,python-mock)
       ("python-iocapture" ,python-iocapture)))
    (home-page "http://github.com/neithere/argh/")
    (synopsis "Argparse wrapper with natural syntax")
    (description "@code{argh} provides an unobtrusive argparse wrapper with
natural syntax.")
    (license license:lgpl3)))

(define-public python2-argh
  (package-with-python2 python-argh))

; FIXME actually run the tests.
(define-public python-pytest-timeout
  (package
    (name "python-pytest-timeout")
    (version "1.2.0")
    (source
      (origin
        (method url-fetch)
        (uri (pypi-uri "pytest-timeout" version))
        (sha256
          (base32
            "1f3wy93637yqp6dw1xmqwrzkswr81fi8qsxxb60755q8y5l337n2"))))
    (build-system python-build-system)
    (native-inputs
     `(("python-pytest-3.0" ,python-pytest-3.0)))
    (home-page "http://bitbucket.org/pytest-dev/pytest-timeout/")
    (synopsis "py.test plugin to abort hanging tests")
    (description "@code{py.test} plugin to abort hanging tests")
    (license license:expat)))

(define-public python2-pytest-timeout
  (package-with-python2 python-pytest-timeout))

(define-public python-watchdog
  (package
    (name "python-watchdog")
    (version "0.8.3")
    (source
      (origin
        (method url-fetch)
        (uri (pypi-uri "watchdog" version))
        (sha256
          (base32
            "0qj1vqszxwfx6d1s66s96jmfmy2j94bywxiqdydh6ikpvcm8hrby"))))
    (build-system python-build-system)
    (propagated-inputs
     `(("python-argh" ,python-argh)
       ("python-pathtools" ,python-pathtools)
       ("python-pyyaml" ,python-pyyaml)))
    (native-inputs
     `(("python-pytest-3.0" ,python-pytest-3.0)
       ("python-pytest-cov" ,python-pytest-cov)
       ("python-pytest-timeout" ,python-pytest-timeout)))
    (home-page "http://github.com/gorakhargosh/watchdog")
    (synopsis "Filesystem event monitoring")
    (description "@code{watchdog} is a library providing filesystem event
monitoring.")
    (license license:asl2.0)))

(define-public python2-watchdog
  (package-with-python2 python-watchdog))

(define-public python-pytest-watch
  (package
    (name "python-pytest-watch")
    (version "4.1.0")
    (source
      (origin
        (method url-fetch)
        (uri (pypi-uri "pytest-watch" version ".zip"))
        (sha256
          (base32
            "14flsdpwrmgsaw3zs97dj4gfbsr7pif4lhwn0z68avafyxpiz519"))))
    (build-system python-build-system)
    (propagated-inputs
     `(("python-colorama" ,python-colorama)
       ("python-docopt" ,python-docopt)
       ("python-pytest" ,python-pytest)
       ("python-watchdog" ,python-watchdog)))
    (native-inputs
     `(("unzip" ,unzip)))
; FIXME depencecies
    (home-page "http://github.com/joeyespo/pytest-watch")
    (synopsis "Local continuous test runner with pytest and watchdog.")
    (description "@code{pytest-watch} is a local continuous test runner with
pytest and watchdog.")
    (properties
     `((python2-variant . ,(delay python2-pytest-watch))))
    (license license:expat)))

(define-public python2-pytest-watch
  (package-with-python2 (strip-python2-variant
                         python-pytest-watch)))

(define-public python-flask-htpasswd
  (package
    (name "python-flask-htpasswd")
    (version "0.3.1")
    (source
      (origin
        (method url-fetch)
        (uri (pypi-uri "flask-htpasswd" version))
        (sha256
          (base32
            "14q1y1y9i9bhabdnwd25jqzc4ljli23smxfyyh8abxz1vq93pxra"))))
    (build-system python-build-system)
    (arguments
     `(#:phases
       (modify-phases %standard-phases
         (add-before 'check 'setenv
           (lambda _
             (mkdir-p "/tmp/foo")
             (setenv "HOME" "/tmp/foo")
             (substitute* "./test_requirements.txt"
               (("coveralls") "")
               (("pytest-watch") ""))
             #t)))))
    (propagated-inputs
     `(("python-flask" ,python-flask)
       ("python-itsdangerous" ,python-itsdangerous)
       ;("python-mock" ,python-mock)
       ("python-passlib" ,python-passlib)))
    (native-inputs
     `(("python-pytest-3.0" ,python-pytest-3.0)
       ("python-pytest-cov" ,python-pytest-cov)
       ("python-pytest-capturelog" ,python-pytest-capturelog)
       ("python-pytest-pep8" ,python-pytest-pep8)
       ;; pep8 and older pytest require python-pytest-cache
       ("python-pytest-cache" ,python-pytest-cache)
       ("python-pytest-pylint" ,python-pytest-pylint)
       ;("python-pytest-watch" ,python-pytest-watch)
       ("python-tox" ,python-tox)))
    (home-page "http://github.com/carsongee/flask-htpasswd")
    (synopsis "Basic authentication support via htpasswd files in Flask applications")
    (description "@code{flask_htpasswd} provides HTTP Basic authentication support via htpasswd files in flask applications.")
    (license license:bsd-3)))

(define-public python2-libmpssexx
  (package
    (name "python2-libmpssexx")
    (version "1.3")
    (source
      (origin
        (method url-fetch)
        (uri (string-append "https://storage.googleapis.com/"
                            "google-code-archive-downloads/v2/"
                            "code.google.com/libmpsse/"
                            "libmpsse-" version ".tar.gz"))
        (sha256
          (base32
            "0jq7nhqq3na8675jnpfcar3pd3dp3adhhc4lw900swkla01a1wh8"))))
    (build-system gnu-build-system)
    (inputs
     `(("libftdi" ,libftdi)
       ("python" ,python-2)))
    (native-inputs
     `(("pkg-config" ,pkg-config)
       ("swig" ,swig)
       ("which" ,which)))
    (arguments
     `(#:tests? #f ; No tests exist.
       #:make-flags
       (list (string-append "CFLAGS=-Wall -fPIC -fno-strict-aliasing -g -O2 "
                            "$(shell pkg-config --cflags libftdi1)"))
       #:phases
       (modify-phases %standard-phases
         (add-after 'unpack 'set-environment-up
           (lambda* (#:key inputs outputs #:allow-other-keys)
             (chdir "src")
             (setenv "PYDEV" (string-append (assoc-ref inputs "python")
                             "/include/python2.7"))
             #t))
         (replace 'install
           (lambda* (#:key outputs make-flags #:allow-other-keys #:rest args)
             (let* ((out (assoc-ref outputs "out"))
                    (out-python (string-append out
                                               "/lib/python2.7/site-packages"))
                    (install (assoc-ref %standard-phases 'install)))
               (install #:make-flags (cons (string-append "PYLIB=" out-python)
                                           make-flags))))))))
    (home-page "https://code.google.com/archive/p/libmpsse/")
    (synopsis "Python library for MPSSE SPI I2C JTAG adapter by FTDI")
    (description "This package provides a library in order to support the
MPSSE adapter by FTDI that can do SPI, I2C, JTAG.")
    (license license:gpl2+)))

(define-public pudb
 (package
  (name "pudb")
  (version "2017.1.4")
  (source
    (origin
      (method url-fetch)
      (uri (pypi-uri "pudb" version))
      (sha256
        (base32
          "12ilz8jrjif7gk2g8w5b0s2mr1cw6zcqh0jz0n4h2bj0118cpj5a"))))
  (build-system python-build-system)
  (arguments
   `(#:tests? #f)) ; Would require terminal
  (propagated-inputs
   `(("python-pygments" ,python-pygments)
     ("python-urwid" ,python-urwid)))
  (home-page "https://github.com/inducer/pudb")
  (synopsis
    "A full-screen, console-based Python debugger")
  (description
    "A full-screen, console-based Python debugger")
  (license #f)))

(define-public python2-smspdu
 (package
  (name "python2-smspdu")
  (version "1.0")
  (source
    (origin
      (method url-fetch)
      (uri (pypi-uri "smspdu" version))
      (sha256
        (base32
          "1yci0y17ylz0p7rigc0zlzjpw73mdxhljpc595jcfaf4qm47jii2"))))
  (build-system python-build-system)
  (arguments
   `(#:python python-2))
  (home-page "http://pypi.python.org/pypi/smspdu")
  (synopsis
    "SMS PDU encoding and decoding, including GSM-0338 character set")
  (description
    "SMS PDU encoding and decoding, including GSM-0338 character set")
  (license #f)))
